@@ -25,6 +25,7 @@
 #define MSTATUS_MPP			(_UL(3) << MSTATUS_MPP_SHIFT)
 #define MSTATUS_FS			_UL(0x00006000)
 #define MSTATUS_XS			_UL(0x00018000)
+#define MSTATUS_VS			_UL(0x01800000)
 #define MSTATUS_MPRV			_UL(0x00020000)
 #define MSTATUS_SUM			_UL(0x00040000)
 #define MSTATUS_MXR			_UL(0x00080000)
@@ -53,6 +54,7 @@
 #define SSTATUS_SPP			MSTATUS_SPP
 #define SSTATUS_FS			MSTATUS_FS
 #define SSTATUS_XS			MSTATUS_XS
+#define SSTATUS_VS			MSTATUS_VS
 #define SSTATUS_SUM			MSTATUS_SUM
 #define SSTATUS_MXR			MSTATUS_MXR
 #define SSTATUS32_SD			MSTATUS32_SD
